wget --http-user=rexster --http-password=rexster http://rexserver:8182/graphs/users/vertices?key=name\&value=pluto -O pluto.json
wget --http-user=rexster --http-password=rexster http://rexserver:8182/graphs/users/vertices/40016/outIds -O relationships.json
